#'  Item Response Ratio
#'
#' The item response ratio measures the impact of the missingness on the total of a variable in the final dataset (or in data sets where already a number of values have been imputed).
#'
#' @author Beat Hulliger - Juan Berdugo
#' @param data (mandatory): A dataframe containing the data to be processed.
#' @param r1ij (mandatory): A matrix containing the response indicators of dataset 1.
#' @param bij (optional): A matrix containing the structurally missingness indicators. bij can be calculated using the function \code{\link[sdap]{smind}}. If the argument bij is missing, the indicator irr is calculated without considering a misingness indicators matrix.
#' @param obsi (optional): A vector with the observations in rij to to be processed. If the argument obs is missing, all observations are processed.
#' @param varj (optional): A vector with the variables (column numbers) to be considered for the calculation. If the argument varj is missing, all variables are considered for the indicator.
#' @param weight (optional): A vector of weights to be considered when calculating the indicator. If no weight vector is given as an argument, the indicator is calculated without considering different weights.
#' @return A list with the following elements: variables (variables), observations (observations), Indicator IMR (imr).
#' @export


iro <- function(data,r1ij,bij,obsi=1:nrow(data),varj=1:ncol(bij),weight)

{

  #obsi <- 1:nrow(bij)
  #varj <- var.rent

  n <- length(obsi)
  p <- length(varj)

  if (missing(weight)) weight <- rep(1,n)

  #Check existence of r1ij

  if (missing(r1ij)) {
    cat("Missing r1ij!\n")
    break
  }


  #Check existence of bij

  if (missing(bij))
    {
    cat("Missing bij!\n")
    break
    }

  #store the size of the arguments

  sizedata <- as.double(dim(data))
  sizebij <- as.double(dim(bij))
  sizer1ij <- as.double(dim(r1ij))


  #check if the sizes of bij and r1ij match

  sizeindicators<- (sizebij+sizer1ij)/2



  if (!identical(sizeindicators, sizedata))
    {
    print("The sizes of the datasets do not match. Please recalculate bij and/or r1ij.")
    break
    }else
    {
      print("Datasets sizes ok")
    }

  yij <- as.matrix(data[obsi,varj])
  class(yij) <- "numeric"
  yij[is.na(yij)] <- 0

  # Calculate denominator of the function. If it is zero, return zero and break.
  if (length(varj)==1)
  {
    denominator <- weighted.mean((1-bij[obsi,varj])*yij,w=weight)
  }else
  {
    denominator <- weighted.mean(apply((1-bij[obsi,varj])*yij,1,sum),w=weight)
  }

  if(denominator==0)
  {
    iro.value <- 0
    return(iro.value)
    break
  }

  # Calculate IRO
  if(length(varj)==1)
  {
    iro.value = weighted.mean((r1ij[obsi,varj]*(1-bij[obsi,varj])*yij),w=weight)
  }else
  {
    iro.value = weighted.mean(apply((r1ij[obsi,varj]*(1-bij[obsi,varj])*yij),1,sum),w=weight)
  }

  iro.value = iro.value / denominator


  irolist<- list(variables = varj, observations = obsi,  iro=iro.value )

  return(irolist)



}


